      INTEGER FUNCTION P01ABFN(IFAIL,IERROR,SRNAME,NREC,REC)
C     MARK 11.5(F77) RELEASE. NAG COPYRIGHT 1986.
C     MARK 13 REVISED. IER-621 (APR 1988).
C     MARK 13B REVISED. IER-668 (AUG 1988).
C
C     P01ABFN is the error-handling routine for the NAG Library.
C
C     P01ABFN either returns the value of IERROR through the routine
C     name (soft failure), or terminates execution of the program
C     (hard failure). Diagnostic messages may be output.
C
C     If IERROR = 0 (successful exit from the calling routine),
C     the value 0 is returned through the routine name, and no
C     message is output
C
C     If IERROR is non-zero (abnormal exit from the calling routine),
C     the action taken depends on the value of IFAIL.
C
C     IFAIL =  1: soft failure, silent exit (i.e. no messages are
C                 output)
C     IFAIL = -1: soft failure, noisy exit (i.e. messages are output)
C     IFAIL =-13: soft failure, noisy exit but standard messages from
C                 P01ABFN are suppressed
C     IFAIL =  0: hard failure, noisy exit
C
C     For compatibility with certain routines included before Mark 12
C     P01ABFN also allows an alternative specification of IFAIL in which
C     it is regarded as a decimal integer with least significant digits
C     cba. Then
C
C     a = 0: hard failure  a = 1: soft failure
C     b = 0: silent exit   b = 1: noisy exit
C
C     except that hard failure now always implies a noisy exit.
C
C     S.Hammarling, M.P.Hooper and J.J.du Croz, NAG Central Office.
C
C     .. Scalar Arguments ..
      INTEGER                  IERROR, IFAIL, NREC
      CHARACTER*(*)            SRNAME
C     .. Array Arguments ..
      CHARACTER*(*)            REC(*)
C     .. Local Scalars ..
      INTEGER                  I, NERR
      CHARACTER*72             MESS
C     .. External Subroutines ..
      EXTERNAL                 P01ABZN, X04AAFN, X04BAFN
C     .. Intrinsic Functions ..
      INTRINSIC                ABS, MOD
C     .. Executable Statements ..
      CONTINUE
C
C     No argument checking.
C
      IF (IERROR.NE.0) THEN
C
C        Abnormal exit from calling routine
C
         IF (IFAIL.EQ.-1 .OR. IFAIL.EQ.0 .OR. IFAIL.EQ.-13 .OR.
     *       (IFAIL.GT.0 .AND. MOD(IFAIL/10,10).NE.0)) THEN
C
C           Noisy exit
C
            CALL X04AAFN(0,NERR)
            DO 20 I = 1, NREC
               CALL X04BAFN(NERR,REC(I))
   20       CONTINUE
            IF (IFAIL.NE.-13) THEN
               WRITE (MESS,FMT=99999) SRNAME, IERROR
               CALL X04BAFN(NERR,MESS)
               IF (ABS(MOD(IFAIL,10)).NE.1) THEN
C
C                 Hard failure
C
                  CALL X04BAFN(NERR,
     *                     ' ** NAG hard failure - execution terminated'
     *                         )
                  CALL P01ABZN
               ELSE
C
C                 Soft failure
C
                  CALL X04BAFN(NERR,
     *                         ' ** NAG soft failure - control returned'
     *                         )
               END IF
            END IF
         END IF
      END IF
C
      P01ABFN = IERROR
C
      RETURN
C
99999 FORMAT (' ** ABNORMAL EXIT from NAG Library routine ',A,': IFAIL',
     *       ' =',I6)
      END
